#include "common.h"

////////////// CONSTANTS ///////////////////

#define CREATURE_TURN_SPEED .8f
#define PLAYER_SPEED 5
#define OBJECT_DESTROY_SPEED 1
#define CREATURE_DESTROY_SPEED 1

////////////// GLOBAL ///////////////////

Monsters gMonsters;
Objects gObjects;
Player *gPlayer = NULL;


Object *object_at_coord(Coord c)
{
	for(Objects::iterator i=gObjects.begin(); i!=gObjects.end(); i++)
		if((*i)->coord == c || ((*i)->pushed && (*i)->target == c))
			return *i;

	return NULL;
}

Creature *creature_at_coord(Coord c)
{
	for(Monsters::iterator i=gMonsters.begin(); i!=gMonsters.end(); i++)
		if((*i)->coord == c || ((*i)->status==CREATURE_WALKING && (*i)->target == c))
			return *i;

	return NULL;
}

Object *object_at_coord_light(Coord c)
{
	for(Objects::iterator i=gObjects.begin(); i!=gObjects.end(); i++)
		if(world_to_map((*i)->pos) == c)
			return *i;

	return NULL;
}

Creature *creature_at_coord_light(Coord c)
{
	for(Monsters::iterator i=gMonsters.begin(); i!=gMonsters.end(); i++)
		if(world_to_map((*i)->pos) == c)
			return *i;

	return NULL;
}

////////////// CREATURE ///////////////////

Creature::Creature(AniMesh *mesh)
	: AniObject(mesh)
{
	status = CREATURE_IDLE;
	SetCoord(Coord(0,0));
	SetOrientation(2);
	is_destroyed = false;
}


bool Creature::IsFree(Coord c)
{
	int tile = gCurMap->Get(c);
	if(tile>=0 && gTileInfo[tile].can_walk && !object_at_coord(c) && !creature_at_coord(c))
		return true;

	return false;
}

void Creature::Render()
{
	float scale=1.f;
	float h2 = h;
	if(is_destroyed)
	{
		scale = max(0.01f, 1.f-destroy);
		h2 += destroy * 360;
	}

	glPushMatrix();
	_glTranslate(pos);
	glRotatef(h2, 0,1,0);
	glScalef(scale, scale, scale);
	AniObject::Render();
	glPopMatrix();
}

void Creature::SetCoord(Coord c)
{
	coord = c;
	pos = map_to_world(c);
}

void Creature::SetOrientation(int o)
{
	ori = o;
	h = orientation_to_angle(o);
}


void Creature::Update(float ticks)
{
	switch(status)
	{
	case CREATURE_WAITING:
		wait -= ticks;
		if(wait<0.f)
			status = CREATURE_IDLE;
		break;

	case CREATURE_WALKING:
		{
//			float frame = fmod(curTime * animSpeed, float(animNb));
			float frame = curTime * animSpeed;
			if(frame>=7 && frame<12)
			{
				pos = map_to_world(coord) + orientation_to_vector(ori) * ((frame-7)*20);
			}
			else if(frame>=12)
				pos = map_to_world(target);

			if(curTime * animSpeed>=16)
			{
				SetAnim(0, 0, 1);
				SetCoord(target);
				status = CREATURE_IDLE;
			}
		}
		break;

	case CREATURE_TURNING:
		turn += ticks * CREATURE_TURN_SPEED;

		if((ori ^ target_ori)&1)
		{
			if((target_ori+1)%4 == ori)
				h = orientation_to_angle(ori) + turn * -90.f; 
			else
				h = orientation_to_angle(ori) + turn * 90.f; 
		}
		else
				h = orientation_to_angle(ori) + turn * 180.f; 

		if(turn>=1.f)
		{
			status = CREATURE_IDLE;
			ori = target_ori;
			h = orientation_to_angle(ori);
		}
		break;

	}

	AniObject::Update(ticks);
}

void Creature::Destroy()
{
	if(!is_destroyed)
	{
		destroy = 0.f;
		is_destroyed=true;
	}
}

////////////// MONSTER ///////////////////

Monster::Monster(AniMesh *mesh)
	: Creature(mesh)
{
}
void Monster::NewTask()
{
	//if(!(rand()%5))
	//{
	//	status = CREATURE_WAITING;
	//	wait = (rand()%20 + 1) * 0.02f;
	//	return;
	//}

	if(IsFree(coord + orientation_to_coord(ori)) && color!=gCurMap->Get(coord + orientation_to_coord(ori), 1))
	{
		target = coord + orientation_to_coord(ori);
		status = CREATURE_WALKING;
		SetAnim(0, 17, 10);
	}
	else
	{
		Object *obj = object_at_coord(coord + orientation_to_coord(ori));
		if(obj && obj->type>=TILE_FILTER_R && obj->type<=TILE_FILTER_B)
		{
			obj->Destroy();
			status = CREATURE_WAITING;
			wait = 1.1f;
			return;
		}

		int b = rand()%2;
		if(IsFree(coord + orientation_to_coord((ori + (b?1:3))%4)))
		{
			target_ori = (ori + (b?1:3))%4;
			turn = 0.f;
			status = CREATURE_TURNING;
		}
		else if(IsFree(coord + orientation_to_coord((ori + (b?3:1))%4)))
		{
			target_ori = (ori + (b?3:1))%4;
			turn = 0.f;
			status = CREATURE_TURNING;
		}
		else if(IsFree(coord + orientation_to_coord((ori + 2)%4)))
		{
			target_ori = (ori + 2)%4;
			turn = 0.f;
			status = CREATURE_TURNING;
		}
	}
}

void Monster::Update(float ticks)
{
	if(is_destroyed)
	{
		destroy += ticks * CREATURE_DESTROY_SPEED;
		if(destroy>=1.f)
		{
			gMonsters.remove(this);
			delete this;
		}
	}
	else
	{
		switch(status)
		{
		case CREATURE_IDLE:
			NewTask();
			break;
		default:
			Creature::Update(ticks);
		}
	}
}

////////////// PLAYER ///////////////////

Player::Player(AniMesh *mesh)
	: Creature(mesh)
{
	color = 4; // no color 4 so won't be destroyed by lights
}

void Player::Update(float ticks)
{
	if(status==CREATURE_WALKING || status==CREATURE_PUSHING)
	{
		walk += ticks * PLAYER_SPEED;

		if(walk>=1.f)
		{
			if(status==CREATURE_PUSHING)
			{
				push_object->pushed = false;
				push_object->SetCoord(push_object->target);
			}
			status = CREATURE_IDLE;
			SetAnim(0, 1, 10);

			SetCoord(target);
		}
		else
		{
			if(status==CREATURE_PUSHING)
				push_object->pos = map_to_world(push_object->coord) + orientation_to_vector(ori) * walk * TILE_SIZE;
			pos = map_to_world(coord) + orientation_to_vector(ori) * walk * TILE_SIZE;
		}
	}

	if(status==CREATURE_IDLE)
	{
		if(gKeys[SDLK_UP] || gKeys[SDLK_DOWN] || gKeys[SDLK_LEFT] || gKeys[SDLK_RIGHT])
		{
			int o;

			if(gKeys[SDLK_UP])
				o = 0;
			else if(gKeys[SDLK_DOWN])
				o = 2;
			else if(gKeys[SDLK_LEFT])
				o = 1;
			else if(gKeys[SDLK_RIGHT])
				o = 3;

			Coord newTarget = coord + orientation_to_coord(o);
			if(IsFree(newTarget))
			{
				SetOrientation(o);
				target = newTarget;
				walk = 0.f;
				status = CREATURE_WALKING;
				SetAnim(1, 6, 10);

			}
			else
			{
				Object *obj = object_at_coord(newTarget);
				Coord pushTarget = newTarget + orientation_to_coord(o);
				if(obj && IsFree(pushTarget))
				{
					SetOrientation(o);
					target = newTarget;
					walk = 0.f;
					status = CREATURE_PUSHING;
					obj->target = pushTarget;
					obj->pushed = true;
					push_object = obj;
					SetAnim(1, 6, 10);
				}
			}
		}
	}
	AniObject::Update(ticks);
}

////////////// OBJECT ///////////////////

Object::Object(int type)
{
	this->type = type;
	SetCoord(Coord(0,0));
	SetOrientation(0);
	pushed=false;
	is_destroyed = false;
}


void Object::SetCoord(Coord c)
{
	coord = c;
	pos = map_to_world(c);
}

void Object::SetOrientation(int o)
{
	ori = o;
	h = orientation_to_angle(o);
}

void Object::Render()
{
	float scale=1.f;
	if(is_destroyed)
	{
		scale = max(0.01f, 1.f-destroy);
		h = destroy * 360;
	}

	glPushMatrix();
	_glTranslate(pos);
	glRotatef(h, 0,1,0);
	glScalef(scale, scale, scale);
	gMeshes[type].Render();
	glPopMatrix();
}

void Object::Update(float ticks)
{
	if(is_destroyed)
	{
		destroy += ticks * OBJECT_DESTROY_SPEED;
		if(destroy>=1.f && !pushed)
		{
			gObjects.remove(this);
			delete this;
		}
	}
}


void Object::Destroy()
{
	if(!is_destroyed)
	{
		destroy = 0.f;
		is_destroyed=true;
	}
}
