#include "common.h"

#define LINE_WIDTH 5

float gColors[4][4]=
{
	{1,1,1,.5},
	{1,0,0,.5},
	{0,1,0,.5},
	{0,0,1,.5},
};



////////////// LIGHTRAY ////////////////
void LightRay::Render()
{
	glColor4fv(gColors[color]);

	glBegin(GL_QUADS);
	_glVertex(map_to_world(start)+Vector(-LINE_WIDTH,50,-LINE_WIDTH));
	_glVertex(map_to_world(start)+Vector(LINE_WIDTH,50,LINE_WIDTH));
	_glVertex(endpos+Vector(LINE_WIDTH,50,LINE_WIDTH));
	_glVertex(endpos+Vector(-LINE_WIDTH,50,-LINE_WIDTH));
	glEnd();
}

////////////// LIGHTTRACER ////////////////

LightTracer::LightTracer()
{
	Clear();
}

void LightTracer::Clear()
{
	sources = total = 0;
}

bool LightTracer::IsFree(Coord c)
{
	int tile = gCurMap->Get(c);
	if(tile>=0 && gTileInfo[tile].light_ok && !object_at_coord_light(c) && !creature_at_coord_light(c))
		return true;

	return false;
}

void LightTracer::ComputeRay(LightRay &ray)
{
	Coord c = ray.start;
	Coord dir = orientation_to_coord(ray.ori);

	for(;;)
	{
		c = c+dir;

		gCurMap->Set(c, 1, ray.color);
		if(!IsFree(c))
		{
			Object *obj = object_at_coord_light(c);
			Creature *creature = creature_at_coord_light(c);
			if(!creature && obj && obj->type==TILE_MIRROR_A)
			{
				rays[total].start = c;
				rays[total].color = ray.color;
				rays[total].ori = (ray.ori + (ray.ori&1 ? 3 : 1)) % 4;
				total++;
			}
			if(!creature && obj && obj->type==TILE_MIRROR_B)
			{
				rays[total].start = c;
				rays[total].color = ray.color;
				rays[total].ori = (ray.ori + (ray.ori&1 ? 1 : 3)) % 4;
				total++;
			}
			if(!creature && obj && ((obj->type==TILE_PRISM_A && ray.ori==0) || (obj->type==TILE_PRISM_C && ray.ori==2)))
			{
				rays[total].start = c;
				rays[total].ori = 1;
				rays[total].color = ray.color;
				total++;
				rays[total].start = c;
				rays[total].ori = 3;
				rays[total].color = ray.color;
				total++;
			}
			if(!creature && obj && ((obj->type==TILE_PRISM_B && ray.ori==3) || (obj->type==TILE_PRISM_D && ray.ori==1)))
			{
				rays[total].start = c;
				rays[total].ori = 0;
				rays[total].color = ray.color;
				total++;
				rays[total].start = c;
				rays[total].ori = 2;
				rays[total].color = ray.color;
				total++;
			}
			if(!creature && obj && obj->type==TILE_FILTER_R && ray.color==0)
			{
				rays[total].start = c;
				rays[total].color = 1;
				rays[total].ori = ray.ori;
				total++;
			}
			if(!creature && obj && obj->type==TILE_FILTER_G && ray.color==0)
			{
				rays[total].start = c;
				rays[total].color = 2;
				rays[total].ori = ray.ori;
				total++;
			}
			if(!creature && obj && obj->type==TILE_FILTER_B && ray.color==0)
			{
				rays[total].start = c;
				rays[total].color = 3;
				rays[total].ori = ray.ori;
				total++;
			}
			if(!creature && obj && obj->type==TILE_FILTER_R && ray.color==1)
				obj->Destroy();
			if(!creature && obj && obj->type==TILE_FILTER_G && ray.color==2)
				obj->Destroy();
			if(!creature && obj && obj->type==TILE_FILTER_B && ray.color==3)
				obj->Destroy();
			if(!creature && obj && obj->type==TILE_BLOCK_R && ray.color==1)
				obj->Destroy();
			if(!creature && obj && obj->type==TILE_BLOCK_G && ray.color==2)
				obj->Destroy();
			if(!creature && obj && obj->type==TILE_BLOCK_B && ray.color==3)
				obj->Destroy();
			if(!creature && obj && obj->type==TILE_BLOCK_W)
				obj->Destroy();

			//if(creature && creature->color==ray.color)
			//	creature->Destroy();

			if(obj)
				ray.endpos = obj->pos;
			else if(creature)
				ray.endpos = creature->pos;
			else
				ray.endpos = map_to_world(c);

			Vector endposfix = map_to_world(c);
			if(ray.ori&1)
				ray.endpos.z = endposfix.z;
			else
				ray.endpos.x = endposfix.x;
			break;
		}
	}

	ray.end = c;
//	ray.endpos = map_to_world(c);
}

void LightTracer::Update()
{
	gCurMap->Clear(1);
	total = sources;
	for(int i=0; i<total && i<(MAX_RAYS-2); i++)
		ComputeRay(rays[i]);

	//int tile = gCurMap->Get(c);
	//if(tile>=0 && gTileInfo[tile].can_walk
}

void LightTracer::Render()
{
	glDisable(GL_LIGHTING);
	glDisable(GL_CULL_FACE);
	glDisable(GL_TEXTURE_2D);

	glBlendFunc(GL_SRC_ALPHA, GL_ONE);
	glEnable(GL_BLEND);

	for(int i=0; i<total; i++)
		rays[i].Render();

	glDisable(GL_BLEND);

	glEnable(GL_LIGHTING);
	glEnable(GL_CULL_FACE);
	glEnable(GL_TEXTURE_2D);

}

void LightTracer::AddSource(Coord c, int ori)
{
	rays[sources].start = c;
	rays[sources].ori = ori;
	rays[sources].color = 0;
	sources++; total++;
}

