#include "common.h"

/////////// Coord ///////////////
//

Coord orientations[4] =
{
	Coord(0,-1),
	Coord(-1,0),
	Coord(0,1),
	Coord(1,0)
};

Coord orientation_to_coord(int ori)
{
	return orientations[ori];
}

Vector orientation_to_vector(int ori)
{
	Coord c = orientation_to_coord(ori);
	return Vector(float(c.x), 0, float(c.y));
}

/////////// Map ///////////////
//

Map::Map()
{
	data = NULL;
}

Map::~Map()
{
	delete[] data;
}

bool Map::Load(const char *filename)
{
	strcpy(this->filename, filename);

	FILE *f = fopen(filename, "rt");
	if(!f)
		return false;

	fscanf(f, "%i %i %i", &width, &height, &layers);
	int size=width*height*layers;
	delete[] data;
	data = new int[size];

	for(int i=0; i<size; i++)
	{
		data[i] = 0;
		fscanf(f, "%i", &data[i]);
	}

	fclose(f);

	return true;
}

void Map::Reload()
{
	Load(filename);
}


bool Map::Save(const char *filename)
{
	FILE *f;
	if(filename)
		f = fopen(filename, "wt");
	else
		f = fopen(this->filename, "wt");

	if(!f)
		return false;

	fprintf(f, "%i %i %i\n", width, height, layers);
	int size=width*height*layers;

	for(int i=0; i<size; i++)
	{
		fprintf(f, " %2i", data[i]);
		if(!((i+1)%width))
			fprintf(f, "\n");

		if(!((i+1)%(width*height)))
			fprintf(f, "\n");
	}

	fclose(f);

	return true;
}

void Map::Render(Coord highlight)
{
	Coord c;
	for(c.x=0; c.x<width; c.x++)
		for(c.y=0; c.y<height; c.y++)
		{
//			if(c==highlight || Get(c,1))
			if(c==highlight)
			{
				glDisable(GL_LIGHTING);
				glColor3f(1.f, .3f, .3f);
			}

			glPushMatrix();

			_glTranslate(map_to_world(c));

			gMeshes[Get(c)].Render();
			glPopMatrix();

			if(c==highlight)
				glEnable(GL_LIGHTING);
		}
}


void Map::Set(Coord c, int tile)
{
	if(c.x>=0 && c.x<width && c.y>=0 && c.y<height)
	{
		data[c.y*width+c.x] = tile;
	}
}

int Map::Get(Coord c)
{
	if(c.x>=0 && c.x<width && c.y>=0 && c.y<height)
		return data[c.x+c.y*width];
	else
		return -1;
}

int Map::Get(Coord c, int l)
{
	if(c.x>=0 && c.x<width && c.y>=0 && c.y<height)
		return data[c.x+c.y*width+l*width*height];
	else
		return -1;
}

void Map::Set(Coord c, int l, int tile)
{
	if(c.x>=0 && c.x<width && c.y>=0 && c.y<height)
	{
		data[c.y*width+c.x+l*width*height] = tile;
	}
}

void Map::Clear(int l)
{
	memset(&data[l*width*height], 0, width*height*sizeof(int));
}
