#ifndef VECTORS_H
#define VECTORS_H

inline float rad(float f) { return f * (2 * float(M_PI) / 360.f); }

class Vector
{
public:
	float x, y, z;

	Vector() { x=y=z=0.f; }
	Vector(float _x, float _y, float _z) { x=_x; y=_y; z=_z; }

	Vector operator*(float f) const
	{
		return Vector(x*f, y*f, z*f);
	}

	Vector operator/(float f) const
	{
		return Vector(x/f, y/f, z/f);
	}

	Vector operator-(const Vector& v) const
	{
		return Vector(x-v.x, y-v.y, z-v.z);
	}

	Vector operator+(const Vector& v) const
	{
		return Vector(x+v.x, y+v.y, z+v.z);
	}

	Vector& operator+=(const Vector& v)
	{
		x += v.x;
		y += v.y;
		z += v.z;
		return *this;
	}

	Vector RotateY(float angle)
	{
		Vector v;

		v.x = cosf(rad(angle)) * x - sinf(rad(angle)) * z;
		v.y = y;
		v.z = cosf(rad(angle)) * z + sinf(rad(angle)) * x;

		return v;
	}

	Vector RotateX(float angle)
	{
		Vector v;

		v.x = x;
		v.y = cosf(rad(angle)) * y + sinf(rad(angle)) * z;
		v.z = cosf(rad(angle)) * z - sinf(rad(angle)) * y;

		return v;
	}

	float Length()
	{
		return sqrtf(x*x+y*y+z*z);
	}

};

inline void _glTranslate(Vector v)
{
	glTranslatef(v.x, v.y, v.z);
}

inline void _glVertex(Vector v)
{
	glVertex3f(v.x, v.y, v.z);
}

inline float dot(Vector v1, Vector v2)
{
	return v1.x*v2.x + v1.y*v2.y + v1.z*v2.z;
}

inline Vector normal(Vector v)
{
	return v / v.Length();
}


Vector plane_ray_intersection(Vector p1, Vector p2, Vector n, float d);

#endif